#ifndef Schrift_H
#define Schrift_H

#include "Critical.h"
#include "Punkt.h"
#include <functional>

namespace Framework
{
    class Bild; //! Bild.h
    class Text; //! Text.h
    class Buchstabe; //! aus dieser Datei
    class Alphabet; //! aus dieser Datei
    class Schrift; //! aus dieser Datei

    //! Speichert die Alphawerte eines Zeichens einer bestimmten Schrift
    //! Die anderen Farbwerte werden durch die Schriftfarbe bestimmt. Daher nur die Alpha werte.
    class Buchstabe
    {
    private:
        Punkt size;
        unsigned char *alpha;
        int schriftSize;
        int ref;

    public:
        //! Erstellt ein neues leeres Zeichnung
        __declspec( dllexport ) Buchstabe();
        //! Lscht das Zeichnung
        __declspec( dllexport ) ~Buchstabe();
        //! Erstellt einen neuen Buchstaben mit bestimmter Gre
        //! \param size Die Gre des Buchstabens in Pixel
        __declspec( dllexport ) void NeuBuchstabe( Punkt &size );
        //! Setzt den Alphawert eines bestimmten Pixels
        //! \param pos Die position des Pixels
        //! \param alpha Der Wert des Pixels
        __declspec( dllexport ) void setPixel( Punkt &pos, unsigned char alpha );
        //! Setzt den Alphawert eines bestimmten Pixels
        //! \param x die x Position des Pixels
        //! \param y die y Position des Pixels
        //! \param alpha Der Wert des Pixels
        __declspec( dllexport ) void setPixel( int x, int y, unsigned char alpha );
        //! Setzt den Alphawert eines bestimmten Pixels
        //! \param i der Index des Pixels. Er berechnet sich durch x + y * breite und geht von 0 bis breite * hhe - 1
        //! \param alpha Der Wert des Pixels
        __declspec( dllexport ) void setPixel( int i, unsigned char alpha );
        //! Setzt die Schriftgre, zu der der Buchstabe gehrt
        //! \param sg Die Schriftgre des Buchstabens.
        __declspec( dllexport ) void setSchriftSize( int sg );
        //! Gibt die Schriftgre zurck, zu der der Buchstabe gehrt
        __declspec( dllexport ) int getSchriftSize() const;
        //! Gibt die alpha Werte des Buchstabens als array zurck wobei die werte Zeilenweise hintereinander stehen
        __declspec( dllexport ) unsigned char *getBuff() const;
        //! Gibt die Gre des Buchstabens in Pixeln nicht skalliert zurck.
        __declspec( dllexport ) const Punkt &getSize() const;
        //! Gibt die Breite des Buchstabens in Pixeln zurck
        __declspec( dllexport ) int getBreite() const;
        //! Gibt die Hhe des Buchstabens in Pixeln zurck
        __declspec( dllexport ) int getHeight() const;
        //! Erhht den Reference Counting Zhler.
        //! \return this.
        __declspec( dllexport ) Buchstabe *getThis();
        //! Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //! \return 0.
        __declspec( dllexport ) Buchstabe *release();
    };

    //! Speichert alle Buchstaben der selben Schriftgre.
    //! Wird von der Schrift klasse verwendet
    class Alphabet
    {
    private:
        Buchstabe **zeichen;
        int schriftSize;
        int ref;

    public:
        //! Erzeugt ein leeres Zeichnung
        __declspec( dllexport ) Alphabet();
        //! Lscht ein Zeichnung
        __declspec( dllexport ) ~Alphabet();
        //! Lscht alle gespeicherten Zeichen
        __declspec( dllexport ) void NeuAlphabet();
        //! Fgt dem Alphabet einen Buchstaben hinzu
        //! Wenn der hinzugefgte Buchstabe bereits existiert wird er berschrieben
        //! \param i Der ASCII code des Buchstaben, der hinzugefgt werden soll
        //! \param buchstabe Der Buchstabe, der hinzugefgt wird
        __declspec( dllexport ) void setBuchstabe( unsigned char i, Buchstabe *buchstabe );
        //! Setzt die Schriftgre des Alphabets und die der gespeicherten buchstaben
        //! \param gr Die Schriftgre des Alphabets
        __declspec( dllexport ) void setSchriftSize( int gr );
        //! Gibt den gespeicherten Buchstaben zu einem bestimmten ASCII Zeichen zurck
        //! \param i Der ASCII code des Zeichens
        //! \return Ein Zeiger zu dem Buchstaben mit erhhtem Reference Counter
        __declspec( dllexport ) Buchstabe *getBuchstabe( unsigned char i ) const;
        //! Gibt den gespeicherten Buchstaben zu einem bestimmten ASCII Zeichen zurck
        //! \param i Der ASCII code des Zeichens
        //! \return Ein Zeiger zu dem Buchstaben ohne erhhtem Reference Counter
        __declspec( dllexport ) Buchstabe *zBuchstabe( unsigned char i ) const;
        //! Prft, ob zu einem bestimmten ASCII code ein Zeichen vorhanden ist
        //! \param b Der zu prfende ASCII code
        //! \return (true), wenn ein Zeichen zu dem Code gefunden wurde. (false) sonnst
        __declspec( dllexport ) bool hatBuchstabe( unsigned char b ) const;
        //! Gibt die Schriftgre zurck, deren Zeichen in diesem Alphabet gespeichert werden
        __declspec( dllexport ) int getSchriftSize() const;
        //! Erhht den Reference Counting Zhler.
        //! \return this.
        __declspec( dllexport ) Alphabet *getThis();
        //! Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //! \return 0.
        __declspec( dllexport ) Alphabet *release();
    };

    //! Specihert eine Liste von Alphabeten mit verschiedener Schriftgre.
    //! Wird von der Schrift Klasse verwendet, um alle Zeichen der Schriftgre nach sortiert zu speichern.
    class AlphabetArray
    {
    private:
        AlphabetArray *next;
        Alphabet *This;

    public:
        //! Erzeugt eine neue Liste
        __declspec( dllexport ) AlphabetArray();
        //! Lscht eine Liste
        __declspec( dllexport ) ~AlphabetArray();

        //! Fgt der Liste ein Alphabet hinzu
        //! Wenn bereits ein Alphabet mit der selben Schriftgre existiert, wird das Alphabet nicht hinzugefgt
        //! \param alphabet Das Alphabet, welches hinzugefgt werden soll
        //! \return (true), wenn das Alphabet hinzugefgt wurde. (false) sonnst.
        __declspec( dllexport ) bool addAlphabet( Alphabet *alphabet );
        //! Lscht ein Alphabet bestimmter Schriftgre aus der Liste
        //! \param sg Die Schriftgre des Alphabets, welches entfernt werden soll
        //! \return (true), wenn ein Alphabet entfernt wurde. (false) sonnst
        __declspec( dllexport ) bool removeAlphabet( int sg );
        //! Setzt den Zeiger auf das nchste Element der Liste auf 0
        __declspec( dllexport ) void setNext0();

        //! Gibt ein bestimmtes Alphabet mit erhhtem Reference Counter zurck
        //! \param sg Die Schriftgre, dessen Alphabet gesucht werden soll
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *getAlphabet( unsigned char sg ) const;
        //! Gibt ein bestimmtes Alphabet ohne erhhtem Reference Counter zurck
        //! \param sg Die Schriftgre, dessen Alphabet gesucht werden soll
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *zAlphabet( unsigned char sg ) const;
        //! Gibt ein bestimmtes Alphabet mit erhhtem Reference Counter zurck
        //! \param index Der Index des gesuchten Alphabets in der Reihenfolge wie sie der Liste hinzugefgt wurden
        //! \param count Hier sollte 0 bergeben werden. Gibt an auf dem wievielten Element der Liste die Funktion aufgerufen wird.
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *getAlphabetI( int index, int count ) const;
        //! Gibt ein bestimmtes Alphabet ohne erhhtem Reference Counter zurck
        //! \param index Der Index des gesuchten Alphabets in der Reihenfolge wie sie der Liste hinzugefgt wurden
        //! \param count Hier sollte 0 bergeben werden. Gibt an auf dem wievielten Element der Liste die Funktion aufgerufen wird.
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *zAlphabetI( int index, int count ) const;
        //! Gibt einen Zeiger auf das nchste Element der Liste zurck
        __declspec( dllexport ) AlphabetArray *getNext() const;
    };

    //! Speichert alle Buchstaben einer Schrift in verschiedenen Schriftgren
    class Schrift
    {
    private:
        unsigned char alphabetAnzahl;
        AlphabetArray *alphabet;
        int ref;

    public:
        //! Erzeugt eine leere Schrift
        __declspec( dllexport ) Schrift();
        //! Lscht ein Zeichnung
        __declspec( dllexport ) ~Schrift();
        //! Fgt der Schrift ein Alphabet hinzu. Sollte bereits ein Alphabet der selben Schriftgre existieren, wird das Alphabet nicht hinzugefgt
        //! \param alphabet Das Alphabet, welches hinzugefgt werden soll
        //! \return (true), wenn das Alphabet hinzugefgt wurde. (false) sonnst
        __declspec( dllexport ) bool addAlphabet( Alphabet *alphabet );
        //! Lscht ein bestimmtes Alphabet aus der Schrift
        //! \param sg Die Schriftgre, deren Alphabet entfernt werden soll
        __declspec( dllexport ) void removeAlphabet( int sg );
        //! Gibt ein bestimmtes Alphabet mit erhhtem Reference Counter zurck
        //! \param sg Die Schriftgre, dessen Alphabet gesucht werden soll
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *getAlphabet( int sg ) const;
        //! Gibt ein bestimmtes Alphabet ohne erhhtem Reference Counter zurck
        //! \param sg Die Schriftgre, dessen Alphabet gesucht werden soll
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *zAlphabet( int sg ) const;
        //! Gibt ein bestimmtes Alphabet mit erhhtem Reference Counter zurck
        //! \param index Der Index des gesuchten Alphabets in der Reihenfolge wie sie der Liste hinzugefgt wurden
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *getAlphabetI( int index ) const;
        //! Gibt ein bestimmtes Alphabet ohne erhhtem Reference Counter zurck
        //! \param index Der Index des gesuchten Alphabets in der Reihenfolge wie sie der Liste hinzugefgt wurden
        //! \return (0), fals kein passendes Alphabet gefunden wurde
        __declspec( dllexport ) Alphabet *zAlphabetI( int index ) const;
        //! Gibt zurck, wie viele Alphabete (und damit Schriftgren) in der Schrift enthalten sind
        __declspec( dllexport ) unsigned char getAlphabetAnzahl() const;
        //! Erhht den Reference Counting Zhler.
        //! \return this.
        __declspec( dllexport ) Schrift *getThis();
        //! Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //! \return 0.
        __declspec( dllexport ) Schrift *release();
    };

    class TextRenderer
    {
    protected:
        Schrift *s;
        int schriftSize;
        int zeilenAbstand;
        int zeichenAbstand;
        int ref;

    public:
        __declspec( dllexport ) TextRenderer();
        __declspec( dllexport ) TextRenderer( Schrift *schrift );
        __declspec( dllexport ) virtual ~TextRenderer();
        __declspec( dllexport ) void setSchriftZ( Schrift *schrift );
        __declspec( dllexport ) Schrift *getSchrift();
        __declspec( dllexport ) Schrift *zSchrift();
        //! Setzt die Schriftgre, in der gezeichnet werden soll. Die Schrift whlt automatisch das passende Alphabet zum Zeichnen
        //! \param sg Die Schriftgre
        __declspec( dllexport ) void setSchriftSize( int sg );
        //! Setzt den Zeilenabstand, der zum zeichnen verwendet werden soll
        //! \param za Der Zeilenabstand zum unteren Ende der darber liegenden zeile in Pixeln
        __declspec( dllexport ) void setZeilenAbstand( int za );
        //! Setzt den Zeichenabstand, der zum zeichnen verwendet werden soll
        //! \param za Der Zeichenabstand zum unteren Ende der darber liegenden zeile in Pixeln
        __declspec( dllexport ) void setZeichenAbstand( int za );
        //! Fgt Zeilenumbrche in den Text ein, so dass er bei einer vorgegebenen Breite follstndig angezeigt wird
        //! \param zText Der text, in den die Zeilenumbrche eingefgt werden sollen
        //! \param maxBreite Die Breite in Pixeln auf der der Text follstndig angezeigt werden soll
        __declspec( dllexport ) virtual void textFormatieren( Text *zText, int maxBreite );
        //! Zeichnet einen Bestimmten Text mit Cursor und einfrbung auf ein Bild
        //! Nutze (setPosition) und (setDrawSchriftGre) um die Position und die Gre zu verndern
        //! \param x x position des ersten zeichens
        //! \param y y position des ersten zeichens
        //! \param txt Der Text, der gezeichnet werden soll
        //! \param zRObj Das Bild, auf das gezeichnet werden soll
        //! \param cpos Die position des Cursors im Text
        //! \param cf Die Farbe des Cursors
        //! \param fbeg Die Position des Zeichens im Text, wo die Einfrbung beginnen soll. Der Text wird von dort bis zur Cursorposition eingefrbt
        //! \param ff Die Hintergrund Farbe des eingefrbten Textes
        //! \param f Eine Funktion die fr jeden Buchstaben aufgerufen wird und seine Farbe zurckgibt
        __declspec( dllexport ) virtual void renderText( int x, int y, const char *txt, Bild &zRObj, std::function< int( int, int, int ) > f, int cpos = -1, int cf = 0, int fbeg = -1, int ff = 0 );
        //! Zeichnet einen Bestimmten Text mit Cursor und einfrbung auf ein Bild
        //! Nutze (setPosition) und (setDrawSchriftGre) um die Position und die Gre zu verndern
        //! \param x x position des ersten zeichens
        //! \param y y position des ersten zeichens
        //! \param txt Der Text, der gezeichnet werden soll
        //! \param zRObj Das Bild, auf das gezeichnet werden soll
        //! \param cpos Die position des Cursors im Text
        //! \param cf Die Farbe des Cursors
        //! \param fbeg Die Position des Zeichens im Text, wo die Einfrbung beginnen soll. Der Text wird von dort bis zur Cursorposition eingefrbt
        //! \param ff Die Hintergrund Farbe des eingefrbten Textes
        //! \param f Die Farbe, in der der Text gezeichnet werden soll
        __declspec( dllexport ) virtual void renderText( int x, int y, const char *txt, Bild &zRObj, int f, int cpos = -1, int cf = 0, int fbeg = -1, int ff = 0 );
        //! Zeichnet einen Bestimmten Buchstaben mit einfrbung auf ein Bild
        //! Nutze (setPosition) und (setDrawSchriftGre) um die Position und die Gre zu verndern
        //! \param x x position des ersten zeichens
        //! \param y y position des ersten zeichens
        //! \param txt Der Text, der gezeichnet werden soll
        //! \param zRObj Das Bild, auf das gezeichnet werden soll
        //! \param color Die Farbe, in der der Text gezeichnet werden soll
        //! \param underlined 1, falls der Text unterstrichen sein soll
        //! \param selected 1, falls das zeichen eingefrbt sein soll
        //! \param selectedBackgroundColor Die Hintergrund Farbe des eingefrbten Textes
        __declspec( dllexport ) virtual void renderChar( int &x, int y, char c, Bild &zRObj, int color, bool underlined = 0, bool selected = 0, int selectedBackgroundColor = 0 );
        //! Gibt die Schriftgre zurck, die zum Zeichnen verwendet wird
        __declspec( dllexport ) int getSchriftSize() const;
        //! Gibt den Abstand in Pixeln zum unteren Ende der darber ligenden Zeile zurck
        __declspec( dllexport ) int getZeilenabstand() const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Text vollstndig darzustellen
        //! \param txt Der Text, von dem die Breite in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getTextBreite( const char *txt ) const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Text vollstndig darzustellen
        //! \param txt Der Text, von dem die Hhe in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getTextHeight( const char *txt ) const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Buchstaben vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Breite in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharWidth( const char c ) const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Text vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Hhe in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharHeight( const char c ) const;
        //! Gibt den Abstand in Pixeln zum unteren Ende der darber ligenden Zeile zurck
        __declspec( dllexport ) int getZeilenAbstand() const;
        //! Gibt den Abstand in Pixeln zum zwischen zwei zeichen auf der x Achse zurck
        __declspec( dllexport ) int getZeichenAbstand() const;
        //! Gibt die skallierte Hhe zurck, die eine gezeichnete Zeile in Pixeln bentigt
        __declspec( dllexport ) int getZeilenHeight() const;
        //! Ermittelt das Zeichen im Text, auf das die Maus zeigt
        //! \param zTxt Der Text, auf den die Maus Zeigt
        //! \param mausX Die X Position der Maus in Pixeln Relativ zur Position des ersten Zeichens
        //! \param mausY Die Y Position der Maus in Pixeln Relativ zur Position des ersten Zeichens
        __declspec( dllexport ) virtual int textPos( const char *txt, int mausX, int mausY ) const;
        //! Erhht den Reference Counting Zhler.
        //! \return this.
        __declspec( dllexport ) TextRenderer *getThis();
        //! Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Objekt automatisch gelscht.
        //! \return 0.
        __declspec( dllexport ) virtual TextRenderer *release();
    };

    class GravurTextRenderer : public TextRenderer
    {
    public:
        __declspec( dllexport ) GravurTextRenderer();
        __declspec( dllexport ) GravurTextRenderer( Schrift *schrift );
        __declspec( dllexport ) virtual ~GravurTextRenderer();
        //! Zeichnet einen Bestimmten Buchstaben mit einfrbung auf ein Bild
        //! Nutze (setPosition) und (setDrawSchriftGre) um die Position und die Gre zu verndern
        //! \param x x position des ersten zeichens
        //! \param y y position des ersten zeichens
        //! \param txt Der Text, der gezeichnet werden soll
        //! \param zRObj Das Bild, auf das gezeichnet werden soll
        //! \param color Die Farbe, in der der Text gezeichnet werden soll
        //! \param underlined 1, falls der Text unterstrichen sein soll
        //! \param selected 1, falls das zeichen eingefrbt sein soll
        //! \param selectedBackgroundColor Die Hintergrund Farbe des eingefrbten Textes
        __declspec( dllexport ) virtual void renderChar( int &x, int y, char c, Bild &zRObj, int color, bool underlined = 0, bool selected = 0, int selectedBackgroundColor = 0 );
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Buchstaben vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Breite in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharWidth( const char c ) const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Text vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Hhe in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharHeight( const char c ) const;
    };

    class KursivTextRenderer : public TextRenderer
    {
    public:
        __declspec( dllexport ) KursivTextRenderer();
        __declspec( dllexport ) KursivTextRenderer( Schrift *schrift );
        __declspec( dllexport ) virtual ~KursivTextRenderer();
        //! Zeichnet einen Bestimmten Buchstaben mit einfrbung auf ein Bild
        //! Nutze (setPosition) und (setDrawSchriftGre) um die Position und die Gre zu verndern
        //! \param x x position des ersten zeichens
        //! \param y y position des ersten zeichens
        //! \param txt Der Text, der gezeichnet werden soll
        //! \param zRObj Das Bild, auf das gezeichnet werden soll
        //! \param color Die Farbe, in der der Text gezeichnet werden soll
        //! \param underlined 1, falls der Text unterstrichen sein soll
        //! \param selected 1, falls das zeichen eingefrbt sein soll
        //! \param selectedBackgroundColor Die Hintergrund Farbe des eingefrbten Textes
        __declspec( dllexport ) virtual void renderChar( int &x, int y, char c, Bild &zRObj, int color, bool underlined = 0, bool selected = 0, int selectedBackgroundColor = 0 );
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Buchstaben vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Breite in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharWidth( const char c ) const;
        //! Ermittelt, wie viele Pixel bentigt werden, um einen Bestimmten Text vollstndig darzustellen
        //! \param c Der Buchstabe, von dem die Hhe in Pixeln ermitelt werden soll
        __declspec( dllexport ) virtual int getCharHeight( const char c ) const;
    };
}

#endif